unit Useparse;
{
Purpose : Basically this unit is responsible for going from a filename to a
          stringlist of units that are used
Author  : Chris R  August 1996
Modified:
}
interface
uses classes,sysutils;
type
   eTParseState = (psInComment,psInBrackets,psInWhite,psInToken,psDone,psError,psInQuotedToken);
{}
   TUsesParser = class
      private
         FFileIn     :textfile;
         FFileName   :TFileName;
         FCurrentLineNo :integer;
         FCurrentLine:string;
         FParseState :eTParseState;
         FCurrentToken :string;
         procedure SetFileIn(const FileName:TFileName);
      public
         constructor create;
         destructor destroy;
         procedure ReadUsesStatement(UnitList:TstringList);
         procedure WriteError(const Errmess:string);
         function  LocateUsesStatement:boolean;
         property FileName:TFileName write SetFileIn;
         property CurrentLineNo:integer read FCurrentLineNo;
      end;

implementation
uses dialogs,forms;
const
  ERRORFILE = 'USEDERRS.TXT';

{}
constructor TUsesParser.Create;
begin
  FFileName := '';  {For first time close flag }
  if fileexists(ERRORFILE) then
    deletefile(ERRORFILE);
end;

{}
destructor TUsesparser.destroy;
begin
  if FFileName>'' then
    Close(FFileIn);
  if fileexists(ERRORFILE) then
    showmessage('Errors are recorded in '+ERRORFILE);
end;

{}
procedure TUsesParser.SetFileIn(const FileName:TFileName);
begin
   if FFileName > '' then
      close(FFileIn);
   FFileName := FileName;
   assign(FFileIn,FileName);
   reset(FFileIn);
   FCurrentLineNo := 0;
   FCurrentLine := '';
   FParseState  := psInWhite;
   FCurrentToken := '';
end; {of SetFileIn}

{}
procedure TUsesParser.ReadUsesStatement(UnitList:TstringList);
var
   ColNo :integer;
   ch :char;
   done :boolean;
   CurrentToken :string;
   procedure SaveCurrentToken;
   begin
     if uppercase(CurrentToken)='IN' then
       UnitList.delete(unitlist.count-1)
     else
       if CurrentToken > '' then
         UnitList.add(CurrentToken);
     CurrentToken := '';
   end; {of saveCurrenttoken}
   function GetChar:char;
   var
      NextChar :char;
   begin
     if colno <= length(FCurrentLine) then
     begin
       result := FCurrentLine[Colno];
       inc(ColNo);
     end
     else
     begin
       if system.eof(FFileIn) then
         result := ';'
       else
       begin
         read(FFileIn,nextChar);
         result := NextChar;
         if NextChar = ^M then
            inc(FCurrentLineNo);
       end;
     end;
   end; {of GetChar}
   {Main Read Uses Statement }
begin
   UnitList.clear;
   FParseState := psInWhite;
   CurrentToken := '';
   done := false;
   colno := 5;        {start walking through the current line after uses}
   while not done do
   begin
     ch := getchar;
     case ch of
     ' ',^J,^M,^I,',' :
       if (FParseState = psInToken) or (FParseState = psInQuotedToken) then
       begin
          SaveCurrentToken;
          FParseState := psInWhite;
       end;
     '{' :
       begin
         if (FParseState = psInToken) or (FParseState = psInQuotedToken) then
           SaveCurrentToken;
         FParseState := psInComment;
       end;
     '}' :
       begin
         if FParseState = psInComment then
           FParseState := psInWhite
         else
           if FParseState = psInQuotedToken then
              CurrentToken := CurrentToken+ch
           else
              FParseState := psError;
       end;
     '(' :
       begin
         if (FParseState = psInToken) or (FParseState = psInQuotedToken) then
           SaveCurrentToken;
         FParseState := psInBrackets;
       end;
     ')' :
       begin
         if FParseState = psInBrackets then
           FParseState := psInWhite
         else
           if FParseState = psInQuotedToken then
              CurrentToken := CurrentToken+ch
           else
              FParseState := psError;
       end;
     '''' : {SingleQuote}
       begin
         if FParseState = psInQuotedToken then
         begin
            SaveCurrentToken;
            FParseState := psInWhite;
         end
         else
         if FParseState = psInWhite then
            FParseState := psInQuotedToken
         else
            if FParseState = psInToken then
               FParseState := psError;
       end;
     ';' :
       begin
         if (FParseState = psInToken) or (FParseState = psInQuotedToken) then
           SaveCurrentToken;
         Done := true;
       end;
     'A'..'Z','a'..'z','0'..'9','_','\','.',':':  {good chars for tokens }
       begin
         if FParseState = psInWhite then
           FParseState := psInToken;
         if (FParseState = psInToken) or (FParseState = psInQuotedToken) then
           CurrentToken := CurrentToken+ch;
       end;
     else   {any rogue characters }
       begin
         if (FParseState = psInToken) or (FParseState = psInQuotedToken) then
         begin
           FParseState := psError;
         end;
       end;
     end; {of case }
     if FParseState = psError then
     begin
       WriteError('Parse error in '+FFileName+' line '+inttostr(FCurrentLineNo));
       UnitList.clear; {dont return stuff from suspect line}
       Done := true;
     end;
   end; {of while not done }
   FCurrentLine := '';
end; {ReadUsesStatement}

{}
function TUsesParser.LocateUsesStatement:boolean;
var
   i,ToDelete :integer;
begin
   result := true;
   while not system.eof(FFileIn) do
     if (uppercase(Copy(FCurrentLine,1,4))='USES') then
        exit
     else
     begin
        Application.ProcessMessages;
        readln(FFileIn,FCurrentLine);
        {Remove left spaces efficiently as possible }
        if (length(FCurrentLine)>0) and (FCurrentLine[1]=' ') then
        begin
           ToDelete := 0;
           for i := 1 to length(FCurrentLine) do
             if FCurrentLine[i]=' ' then
                inc(ToDelete)
             else
                break;
           if ToDelete>0 then
              Delete(FCurrentLine,1,ToDelete);
        end;
        inc(FCurrentLineNo);
     end;
   {must have reach end of file so we havent located it}
   result := false;
end; {of LocateUsesStatement}

{}
procedure TUsesParser.WriteError(const Errmess:string);
var ErrLog :textfile;
begin
  assign(ErrLog,ERRORFILE);
  {$i-}
  append(ErrLog);
  {$i+}
  if IOresult <>0 then
    rewrite(ErrLog);
  writeln(ErrLog,ErrMess);
  closefile(ErrLog);
end; {of WriteError }

end.
